/* $Id: replay_controller.hpp 49117 2011-04-05 19:39:56Z anonymissimus $ */
/*
   Copyright (C) 2006 - 2011 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
   wesnoth playlevel Copyright (C) 2003 by David White <dave@whitevine.net>
   Part of the Battle for Wesnoth Project http://www.wesnoth.org/

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2
   or at your option any later version.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY.

   See the COPYING file for more details.
*/

#ifndef REPLAY_CONTROLLER_H_INCLUDED
#define REPLAY_CONTROLLER_H_INCLUDED

#include "game_end_exceptions.hpp"
#include "gamestatus.hpp"
#include "play_controller.hpp"

#include <vector>

class video;

class replay_controller : public play_controller
{
public:
	replay_controller(const config& level, game_state& state_of_game,
		const int ticks, const int num_turns, const config& game_config, CVideo& video);
	virtual ~replay_controller();

	virtual bool can_execute_command(hotkey::HOTKEY_COMMAND command, int index=-1) const;

	//event handlers
	virtual void preferences();
	virtual void show_statistics();
	void play_replay();
	void reset_replay();
	void stop_replay();
	void replay_next_turn();
	void replay_next_side();
	void process_oos(const std::string& msg) const;
	void replay_show_everything();
	void replay_show_each();
	void replay_show_team1();
	void replay_skip_animation();

	virtual void force_end_turn() {}
	virtual void force_end_level(LEVEL_RESULT) {}
	virtual void check_end_level() {}

	std::vector<team> teams_start_;

protected:
	virtual void init_gui();

private:
	void init();
	virtual void play_turn();
	virtual void play_side(const unsigned int team_index, bool save);
	void update_teams();
	void update_gui();
	void init_replay_display();
	void rebuild_replay_theme();
	void handle_generic_event(const std::string& name);

	game_state gamestate_start_;
	unit_map units_start_;
	tod_manager tod_manager_start_;

	unsigned int current_turn_;
	int delay_;
	bool is_playing_;

	bool show_everything_;
	unsigned int show_team_;
	
	class replay_buttons_wrapper {
	
		bool is_valid_;
		
		//bit mask
		unsigned enabled_buttons_;
		
		void update_buttons_states();
	
	public:
		static const unsigned PLAY_BUTTON_ENABLED = 1;
		static const unsigned STOP_BUTTON_ENABLED = 2;
		static const unsigned RESET_BUTTON_ENABLED = 4;
		static const unsigned PLAY_TURN_BUTTON_ENABLED = 8;
		static const unsigned PLAY_SIDE_BUTTON_ENABLED = 16;
		
		replay_buttons_wrapper() : is_valid_(false), enabled_buttons_() {}
	
		gui::button* play_button_;
		gui::button* stop_button_;
		gui::button* reset_button_;
		gui::button* play_turn_button_;
		gui::button* play_side_button_;

		void update(boost::scoped_ptr<game_display>& gui_);
		void playback_should_start();
		void playback_should_stop(bool is_playing);

		void reset_buttons();
	};
	
	replay_buttons_wrapper buttons_;
};


LEVEL_RESULT play_replay_level(const config& terrain_config,
		const config* level, CVideo& video,
		game_state& state_of_game);

#endif
